@@ -24,7 +24,7 @@
 #include <QtGui/QPainter>
 #include <QtGui/QKeyEvent>
 #include <QtGui/QAction>
-#include <QtGui/QItemDelegate>
+#include <QtGui/QStyledItemDelegate>
 #include <QtGui/QHeaderView>
 
 #include <klocale.h>
@@ -38,7 +38,7 @@
 #include "kateextendedattribute.h"
 
 //BEGIN KateStyleTreeDelegate
-class KateStyleTreeDelegate : public QItemDelegate
+class KateStyleTreeDelegate : public QStyledItemDelegate
 {
   public:
     KateStyleTreeDelegate(KateStyleTreeWidget* widget);
@@ -367,16 +367,16 @@ void KateStyleTreeDelegate::paint( QPainter* painter, const QStyleOptionViewItem
       styleContextItem.palette.setBrush(QPalette::HighlightedText, brush);
     }
 	
-    return QItemDelegate::paint(painter, styleContextItem, index);
+    return QStyledItemDelegate::paint(painter, styleContextItem, index);
   }
   
   if (!columns.contains(index.column())) {
-    return QItemDelegate::paint(painter, option, index);
+    return QStyledItemDelegate::paint(painter, option, index);
   }
 
   QVariant displayData = index.model()->data(index);
   if (displayData.type() != QVariant::Brush)
-    return QItemDelegate::paint(painter, option, index);
+    return QStyledItemDelegate::paint(painter, option, index);
 
   QBrush brush = qVariantValue<QBrush>(displayData);
 
@@ -387,7 +387,7 @@ void KateStyleTreeDelegate::paint( QPainter* painter, const QStyleOptionViewItem
   bool set = brush != QBrush();
 
   if (!set) {
-    opt.text = i18nc("No text or background colour set", "None set");
+    opt.text = i18nc("No text or background color set", "None set");
     brush = Qt::white;
   }
   
@@ -440,6 +440,10 @@ void KateStyleTreeWidgetItem::initStyle()
   setFlags(Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
 }
 
+static Qt::CheckState toCheckState(bool b) {
+    return b ? Qt::Checked : Qt::Unchecked;
+}
+
 QVariant KateStyleTreeWidgetItem::data( int column, int role ) const
 {
   if (column == Context) {
@@ -463,24 +467,25 @@ QVariant KateStyleTreeWidgetItem::data( int column, int role ) const
   if (role == Qt::CheckStateRole) {
     switch (column) {
       case Bold:
-        return style()->fontBold();
+        return toCheckState(style()->fontBold());
       case Italic:
-        return style()->fontItalic();
+        return toCheckState(style()->fontItalic());
       case Underline:
-        return style()->fontUnderline();
+        return toCheckState(style()->fontUnderline());
       case StrikeOut:
-        return style()->fontStrikeOut();
+        return toCheckState(style()->fontStrikeOut());
       case UseDefaultStyle:
         /* can't compare all attributes, currentStyle has always more than defaultStyle (e.g. the item's name),
          * so we just compare the important ones:*/
-        return currentStyle->foreground() == defaultStyle->foreground()
+        return toCheckState(
+               currentStyle->foreground() == defaultStyle->foreground()
             && currentStyle->background() == defaultStyle->background()
             && currentStyle->selectedForeground() == defaultStyle->selectedForeground()
             && currentStyle->selectedBackground() == defaultStyle->selectedBackground()
             && currentStyle->fontBold() == defaultStyle->fontBold()
             && currentStyle->fontItalic() == defaultStyle->fontItalic()
             && currentStyle->fontUnderline() == defaultStyle->fontUnderline()
-            && currentStyle->fontStrikeOut() == defaultStyle->fontStrikeOut();
+            && currentStyle->fontStrikeOut() == defaultStyle->fontStrikeOut());
     }
   }
 
@@ -628,7 +633,7 @@ void KateStyleTreeWidgetItem::setColor( int column )
   else if ( column == SelectedForeground )
   {
     c = currentStyle->selectedForeground().color();
-    d = currentStyle->selectedForeground().color();
+    d = defaultStyle->selectedForeground().color();
   }
   else if ( column == Background )
   {
