@@ -101,4 +101,12 @@ bool KToggleFullScreenAction::eventFilter( QObject* object, QEvent* event )
   return false;
 }
 
+void KToggleFullScreenAction::setFullScreen( QWidget* window, bool set )
+{
+  if( set )
+    window->setWindowState( window->windowState() | Qt::WindowFullScreen );
+  else
+    window->setWindowState( window->windowState() & ~Qt::WindowFullScreen );
+}
+
 #include "ktogglefullscreenaction.moc"
