@@ -502,7 +502,7 @@ void KWindowSystem::setOnDesktop( WId win, int desktop )
         x -= w / 2; // from center back to topleft
         y -= h / 2;
         p = constrainViewportRelativePosition( QPoint( x, y ));
-        int flags = ( 0x20 << 12 ) | ( 0x03 << 8 ) | 10; // from tool(?), x/y, static gravity
+        int flags = ( NET::FromTool << 12 ) | ( 0x03 << 8 ) | 10; // from tool(?), x/y, static gravity
         KWindowSystemPrivate* const s_d = s_d_func();
         s_d->moveResizeWindowRequest( win, flags, p.x(), p.y(), w, h );
         return;
@@ -557,8 +557,7 @@ WId KWindowSystem::transientFor( WId win )
 
 void KWindowSystem::setMainWindow( QWidget* subwindow, WId mainwindow )
 {
-    // This will be Qt::WA_X11BypassTransientForHint in Qt4.4, but the code is already there now.
-    subwindow->setAttribute( Qt::WidgetAttribute( 99 ));
+    subwindow->setAttribute( Qt::WA_X11BypassTransientForHint );
     if( mainwindow != 0 )
         XSetTransientForHint( QX11Info::display(), subwindow->winId(), mainwindow );
     else
@@ -690,6 +689,8 @@ void KWindowSystem::setIcons( WId win, const QPixmap& icon, const QPixmap& miniI
     if ( miniIcon.isNull() )
 	return;
     img = miniIcon.toImage().convertToFormat( QImage::Format_ARGB32 );
+    if ( img.isNull() )
+        return;
     ni.size.width = img.size().width();
     ni.size.height = img.size().height();
     ni.data = (unsigned char *) img.bits();
@@ -793,7 +794,7 @@ QRect KWindowSystem::workArea( const QList<WId>& exclude, int desktop )
         desktop = s_d->currentDesktop();
 
     QList<WId>::ConstIterator it1;
-    for( it1 = s_d->windows.begin(); it1 != s_d->windows.end(); ++it1 ) {
+    for( it1 = s_d->windows.constBegin(); it1 != s_d->windows.constEnd(); ++it1 ) {
 
         if(exclude.contains(*it1))
             continue;
@@ -965,6 +966,13 @@ void KWindowSystem::doNotManage( const QString& title )
         .call("doNotManage", title);
 }
 
+void KWindowSystem::allowExternalProcessWindowActivation( int pid )
+{
+    // Normally supported by X11, but may depend on some window managers ?
+    Q_UNUSED(pid)
+}
+
+
 bool KWindowSystem::mapViewport()
 {
     KWindowSystemPrivate* const s_d = s_d_func();
